@@ -1,50 +1,52 @@
-context("Updating dates")
-
 test_that("update.Date returns a date object", {
   date <- as.Date("05/05/2010", "%m/%d/%Y")
-  expect_is(update(date, days = 1), "Date")
-  expect_is(update(date, ydays = 1), "Date")
-  expect_is(update(date, mdays = 1), "Date")
-  expect_is(update(date, wdays = 1), "Date")
-  expect_is(update(date, months = 1), "Date")
-  expect_is(update(date, years = 2001), "Date")
-  expect_is(update(date, tzs = "UTC"), "Date")
+  expect_s3_class(update(date, days = 1), "Date")
+  expect_s3_class(update(date, ydays = 1), "Date")
+  expect_s3_class(update(date, mdays = 1), "Date")
+  expect_s3_class(update(date, wdays = 1), "Date")
+  expect_s3_class(update(date, months = 1), "Date")
+  expect_s3_class(update(date, years = 2001), "Date")
+  ## expect_s3_class(update(date, tzs = "UTC"), "POSIXct")
+  tz(date) <- "UTC"
+  expect_s3_class(date, "POSIXct")
 })
 
 test_that("update.Date returns a posix object if time is manipulated", {
   date <- as.Date("05/05/2010", "%m/%d/%Y")
-  expect_is(update(date, seconds = 1), "POSIXt")
-  expect_is(update(date, minutes = 1), "POSIXt")
-  expect_is(update(date, hours = 1), "POSIXt")
+  expect_s3_class(update(date, seconds = 1), "POSIXt")
+  expect_s3_class(update(date, minutes = 1), "POSIXt")
+  expect_s3_class(update(date, hours = 1), "POSIXt")
 })
 
 test_that("update.POSIXlt returns a POSIXlt object", {
   poslt <- as.POSIXlt("2010-02-03 13:45:59", tz = "GMT", format = "%Y-%m-%d %H:%M:%S")
-  expect_is(update(poslt, seconds = 1), "POSIXlt")
-  expect_is(update(poslt, minutes = 1), "POSIXlt")
-  expect_is(update(poslt, hours = 1), "POSIXlt")
-  expect_is(update(poslt, days = 1), "POSIXlt")
-  expect_is(update(poslt, ydays = 1), "POSIXlt")
-  expect_is(update(poslt, mdays = 1), "POSIXlt")
-  expect_is(update(poslt, wdays = 1), "POSIXlt")
-  expect_is(update(poslt, months = 1), "POSIXlt")
-  expect_is(update(poslt, years = 2001), "POSIXlt")
-  expect_is(update(poslt, tzs = "UTC"), "POSIXlt")
+  expect_s3_class(update(poslt, seconds = 1), "POSIXlt")
+  expect_s3_class(update(poslt, minutes = 1), "POSIXlt")
+  expect_s3_class(update(poslt, hours = 1), "POSIXlt")
+  expect_s3_class(update(poslt, days = 1), "POSIXlt")
+  expect_s3_class(update(poslt, ydays = 1), "POSIXlt")
+  expect_s3_class(update(poslt, mdays = 1), "POSIXlt")
+  expect_s3_class(update(poslt, wdays = 1), "POSIXlt")
+  expect_s3_class(update(poslt, months = 1), "POSIXlt")
+  expect_s3_class(update(poslt, years = 2001), "POSIXlt")
+  expect_s3_class(update(poslt, tzs = "UTC"), "POSIXlt")
 })
 
 test_that("update.POSIXct returns a POSIXct object", {
-  posct <- as.POSIXct("2010-02-03 13:45:59", tz = "GMT", format
-     = "%Y-%m-%d %H:%M:%S")
-  expect_is(update(posct, seconds = 1), "POSIXct")
-  expect_is(update(posct, minutes = 1), "POSIXct")
-  expect_is(update(posct, hours = 1), "POSIXct")
-  expect_is(update(posct, days = 1), "POSIXct")
-  expect_is(update(posct, ydays = 1), "POSIXct")
-  expect_is(update(posct, mdays = 1), "POSIXct")
-  expect_is(update(posct, wdays = 1), "POSIXct")
-  expect_is(update(posct, months = 1), "POSIXct")
-  expect_is(update(posct, years = 2001), "POSIXct")
-  expect_is(update(posct, tzs = "UTC"), "POSIXct")
+  posct <- as.POSIXct("2010-02-03 13:45:59",
+    tz = "GMT", format
+    = "%Y-%m-%d %H:%M:%S"
+  )
+  expect_s3_class(update(posct, seconds = 1), "POSIXct")
+  expect_s3_class(update(posct, minutes = 1), "POSIXct")
+  expect_s3_class(update(posct, hours = 1), "POSIXct")
+  expect_s3_class(update(posct, days = 1), "POSIXct")
+  expect_s3_class(update(posct, ydays = 1), "POSIXct")
+  expect_s3_class(update(posct, mdays = 1), "POSIXct")
+  expect_s3_class(update(posct, wdays = 1), "POSIXct")
+  expect_s3_class(update(posct, months = 1), "POSIXct")
+  expect_s3_class(update(posct, years = 2001), "POSIXct")
+  expect_s3_class(update(posct, tzs = "UTC"), "POSIXct")
 })
 
 test_that("update.Date performs simple operation as expected", {
@@ -102,7 +104,6 @@ test_that("update.POSIXt performs simple operation as expected", {
 })
 
 test_that("update.POSIXt works on wdays", {
-
   date <- ymd("2017-05-07") ## sunday
   ct <- as.POSIXct("2010-02-03 13:45:59", tz = "America/New_York", format = "%Y-%m-%d %H:%M:%S") ## Wednesday
   expect_equal(wday(update(ct, wdays = 1)), 1)
@@ -143,32 +144,50 @@ test_that("update.POSIXt works on wdays", {
   expect_equal(wday(update(date, wdays = 2, week_start = ws), week_start = ws), 2)
   expect_equal(wday(update(date, wdays = 5, week_start = ws), week_start = ws), 5)
   expect_equal(wday(update(date, wdays = 7, week_start = ws), week_start = ws), 7)
-
 })
 
 test_that("updates on ydays works correctly with leap years", {
-  expect_equal(update(ymd("15-02-03", tz = "UTC"), years = 2000, ydays = 1),
-               ymd("2000-01-01", tz = "UTC"))
-  expect_equal(update(ymd("15-02-03", tz = "UTC"), years = 2015, ydays = 1),
-               ymd("2015-01-01", tz = "UTC"))
-  expect_equal(update(ymd("15-02-03", tz = "UTC"), years = 2016, ydays = 10),
-               ymd("2016-01-10", tz = "UTC"))
-  expect_equal(update(ymd("15-02-03", tz = "America/New_York"), years = 2000, ydays = 1),
-               ymd("2000-01-01", tz = "America/New_York"))
-  expect_equal(update(ymd("15-02-03", tz = "America/New_York"), years = 2015, ydays = 1),
-               ymd("2015-01-01", tz = "America/New_York"))
-  expect_equal(update(ymd("15-02-03", tz = "America/New_York"), years = 2016, ydays = 10),
-               ymd("2016-01-10", tz = "America/New_York"))
-
-  expect_equal(update(ymd(c("2016-02-29", "2016-03-01")), ydays = 1),
-               ymd(c("2016-01-01", "2016-01-01")))
-  expect_equal(update(ymd(c("2016-02-29", "2016-03-01"), tz = "America/New_York"), ydays = 1),
-               ymd(c("2016-01-01", "2016-01-01"), tz = "America/New_York"))
-  expect_equal(update(ymd_hms(c("2016-02-29 1:2:3", "2016-03-01 10:20:30")), ydays = 1),
-               ymd_hms(c("2016-01-01 1:2:3", "2016-01-01 10:20:30")))
-  expect_equal(update(ymd_hms(c("2016-02-29 1:2:3", "2016-03-01 10:20:30"), tz = "America/New_York"), ydays = 1),
-               ymd_hms(c("2016-01-01 1:2:3", "2016-01-01 10:20:30"), tz = "America/New_York"))
+  expect_equal(
+    update(ymd("15-02-03", tz = "UTC"), years = 2000, ydays = 1),
+    ymd("2000-01-01", tz = "UTC")
+  )
+  expect_equal(
+    update(ymd("15-02-03", tz = "UTC"), years = 2015, ydays = 1),
+    ymd("2015-01-01", tz = "UTC")
+  )
+  expect_equal(
+    update(ymd("15-02-03", tz = "UTC"), years = 2016, ydays = 10),
+    ymd("2016-01-10", tz = "UTC")
+  )
+  expect_equal(
+    update(ymd("15-02-03", tz = "America/New_York"), years = 2000, ydays = 1),
+    ymd("2000-01-01", tz = "America/New_York")
+  )
+  expect_equal(
+    update(ymd("15-02-03", tz = "America/New_York"), years = 2015, ydays = 1),
+    ymd("2015-01-01", tz = "America/New_York")
+  )
+  expect_equal(
+    update(ymd("15-02-03", tz = "America/New_York"), years = 2016, ydays = 10),
+    ymd("2016-01-10", tz = "America/New_York")
+  )
 
+  expect_equal(
+    update(ymd(c("2016-02-29", "2016-03-01")), ydays = 1),
+    ymd(c("2016-01-01", "2016-01-01"))
+  )
+  expect_equal(
+    update(ymd(c("2016-02-29", "2016-03-01"), tz = "America/New_York"), ydays = 1),
+    ymd(c("2016-01-01", "2016-01-01"), tz = "America/New_York")
+  )
+  expect_equal(
+    update(ymd_hms(c("2016-02-29 1:2:3", "2016-03-01 10:20:30")), ydays = 1),
+    ymd_hms(c("2016-01-01 1:2:3", "2016-01-01 10:20:30"))
+  )
+  expect_equal(
+    update(ymd_hms(c("2016-02-29 1:2:3", "2016-03-01 10:20:30"), tz = "America/New_York"), ydays = 1),
+    ymd_hms(c("2016-01-01 1:2:3", "2016-01-01 10:20:30"), tz = "America/New_York")
+  )
 })
 
 
@@ -239,7 +258,8 @@ test_that("update performs consecutive roll overs correctly for
   Date objects regardless of order", {
   date <- update(as.Date("11/01/2010", "%m/%d/%Y"),
     months = 13, days = 32, hours = 25, minutes = 61, seconds
-     = 61)
+    = 61
+  )
   expect_equal(second(date), 1)
   expect_equal(minute(date), 2)
   expect_equal(hour(date), 2)
@@ -250,7 +270,8 @@ test_that("update performs consecutive roll overs correctly for
   expect_equal(year(date), 2011)
   expect_match(tz(date), "UTC")
   date2 <- update(as.Date("11/01/2010", "%m/%d/%Y"),
-    seconds = 61, minutes = 61, hours = 25, days = 32, months = 13)
+    seconds = 61, minutes = 61, hours = 25, days = 32, months = 13
+  )
   expect_equal(second(date2), 1)
   expect_equal(minute(date2), 2)
   expect_equal(hour(date2), 2)
@@ -264,10 +285,14 @@ test_that("update performs consecutive roll overs correctly for
 
 test_that("update performs consecutive roll overs correctly for
   POSIXlt objects", {
-  posl <- as.POSIXlt("2010-11-01 00:00:00", tz = "GMT", format
-     = "%Y-%m-%d %H:%M:%S")
-  poslt <- update(posl, months = 13, days = 32, hours = 25,
-    minutes = 61, seconds = 61)
+  posl <- as.POSIXlt("2010-11-01 00:00:00",
+    tz = "GMT", format
+    = "%Y-%m-%d %H:%M:%S"
+  )
+  poslt <- update(posl,
+    months = 13, days = 32, hours = 25,
+    minutes = 61, seconds = 61
+  )
 
   expect_equal(second(poslt), 1)
   expect_equal(minute(poslt), 2)
@@ -279,8 +304,10 @@ test_that("update performs consecutive roll overs correctly for
   expect_equal(year(poslt), 2011)
   expect_match(tz(poslt), "GMT")
 
-  poslt2 <- update(posl, seconds = 61, minutes = 61, hours = 25,
-    days = 32, months = 13)
+  poslt2 <- update(posl,
+    seconds = 61, minutes = 61, hours = 25,
+    days = 32, months = 13
+  )
 
   expect_equal(second(poslt2), 1)
   expect_equal(minute(poslt2), 2)
@@ -294,10 +321,14 @@ test_that("update performs consecutive roll overs correctly for
 })
 
 test_that("update performs consecutive roll overs correctly for POSIXct objects", {
-  posc <- as.POSIXct("2010-11-01 00:00:00", tz = "GMT", format
-     = "%Y-%m-%d %H:%M:%S")
-  posct <- update(posc, months = 13, days = 32, hours = 25,
-    minutes = 61, seconds = 61)
+  posc <- as.POSIXct("2010-11-01 00:00:00",
+    tz = "GMT", format
+    = "%Y-%m-%d %H:%M:%S"
+  )
+  posct <- update(posc,
+    months = 13, days = 32, hours = 25,
+    minutes = 61, seconds = 61
+  )
 
   expect_equal(second(posct), 1)
   expect_equal(minute(posct), 2)
@@ -309,8 +340,10 @@ test_that("update performs consecutive roll overs correctly for POSIXct objects"
   expect_equal(year(posct), 2011)
   expect_match(tz(posct), "GMT")
 
-  posct2 <- update(posc, seconds = 61, minutes = 61, hours = 25,
-    days = 32, months = 13)
+  posct2 <- update(posc,
+    seconds = 61, minutes = 61, hours = 25,
+    days = 32, months = 13
+  )
 
   expect_equal(second(posct2), 1)
   expect_equal(minute(posct2), 2)
@@ -344,9 +377,39 @@ test_that("update returns NA for date-times in the spring dst gap", {
   expect_true(is.na(update(poslt, tzs = "America/New_York")))
 })
 
+test_that("update handles repeated DST transitions", {
+  repref <- ymd_hms("2022-10-30 01:00:00", tz = "Europe/Amsterdam")
+  rep <- ymd_hms("2022-10-29 00:01:02", tz = "Europe/Amsterdam")
+
+  expect_equal(update(rep, days = 30, hours = 2, minutes = 3, roll_dst = "pre"),
+    repref + 3600 + 3*60 + 2)
+  expect_equal(update(rep, days = 30, hours = 2, minutes = 3, roll_dst = "boundary"),
+    repref + 2*3600)
+  expect_equal(update(rep, days = 30, hours = 2, minutes = 3, roll_dst = "post"),
+    repref + 2*3600 + 3*60 + 2)
+  expect_equal(update(rep, days = 30, hours = 2, seconds = 3.35, roll_dst = "pre"),
+    repref + 3600 + 60 + 3.35)
+  expect_equal(update(rep, days = 30, hours = 2, seconds = 3.35, roll_dst = "boundary"),
+    repref + 2*3600)
+  expect_equal(update(rep, days = 30, hours = 2, seconds = 3.35, roll_dst = "post"),
+    repref + 2*3600 + 60 + 3.35)
+
+  ## our default roll_dst is post
+  repref <- ymd_hms("2022-10-30 01:00:00", tz = "Europe/Amsterdam")
+  rep <- ymd_hms("2022-10-29 00:10:02.35", tz = "Europe/Amsterdam")
+  expect_equal(update(rep, days = 30, hours = 2, minutes = 3),
+    repref + 2*3600 + 3*60 + 2.35)
+  expect_equal(update(rep, days = 30, hours = 2, seconds = 5.6),
+    repref + 2*3600 + 10*60 + 5.6)
+  expect_equal(update(rep, days = 30, hours = 2, seconds = 65.6),
+    repref + 2*3600 + 11*60 + 5.6)
+
+})
+
 test_that("update handles vectors of dates", {
   poslt <- as.POSIXlt(c("2010-02-14 01:59:59", "2010-02-15 01:59:59", "2010-02-16 01:59:59"),
-                      tz = "UTC", format = "%Y-%m-%d %H:%M:%S")
+    tz = "UTC", format = "%Y-%m-%d %H:%M:%S"
+  )
   posct <- as.POSIXct(poslt)
   date <- as.Date(poslt)
   expect_equal(second(update(poslt, seconds = 1)), c(1, 1, 1))
@@ -374,6 +437,7 @@ test_that("update handles single vector of inputs", {
 })
 
 test_that("update handles conformable vectors of inputs", {
+  ## FIXME: these actually should be an error in conformance with vctrs semantics
   poslt <- as.POSIXlt("2010-03-10 01:59:59", tz = "UTC", format = "%Y-%m-%d %H:%M:%S")
   posct <- as.POSIXct(poslt)
   date <- as.Date(poslt)
@@ -393,16 +457,26 @@ test_that("update.POSIXlt returns input of length zero when given input of lengt
 })
 
 test_that("update correctly handles 60 seconds on 59 minute (bug #313)", {
-  expect_equal(ymd_hms("2015-01-01 00:59:00") + seconds(60),
-               ymd_hms("2015-01-01 01:00:00"))
-  expect_equal(ymd_hms("2015-01-01 01:59:00") + seconds(60),
-               ymd_hms("2015-01-01 02:00:00"))
-  expect_equal(ymd_hms("2015-01-01 23:59:00") + seconds(60),
-               ymd_hms("2015-01-02 00:00:00"))
-  expect_equal(ymd_hms("2015-01-01 00:59:05") + seconds(55),
-               ymd_hms("2015-01-01 01:00:00"))
-  expect_equal(ymd_hms("2015-01-01 00:59:59") + seconds(1),
-               ymd_hms("2015-01-01 01:00:00"))
+  expect_equal(
+    ymd_hms("2015-01-01 00:59:00") + seconds(60),
+    ymd_hms("2015-01-01 01:00:00")
+  )
+  expect_equal(
+    ymd_hms("2015-01-01 01:59:00") + seconds(60),
+    ymd_hms("2015-01-01 02:00:00")
+  )
+  expect_equal(
+    ymd_hms("2015-01-01 23:59:00") + seconds(60),
+    ymd_hms("2015-01-02 00:00:00")
+  )
+  expect_equal(
+    ymd_hms("2015-01-01 00:59:05") + seconds(55),
+    ymd_hms("2015-01-01 01:00:00")
+  )
+  expect_equal(
+    ymd_hms("2015-01-01 00:59:59") + seconds(1),
+    ymd_hms("2015-01-01 01:00:00")
+  )
 })
 
 test_that("Updateing on seconds doesn't affect hours", {
@@ -413,6 +487,17 @@ test_that("Updateing on seconds doesn't affect hours", {
   expect_equal(hour(tt), hour(tt2))
 })
 
+test_that("Subtracting months to March 1 produces correct results", {
+  ## #1037
+  time <- ymd("2022-04-01", tz = "America/New_York")
+  expect_equal(time - months(1), ymd("2022-03-01", tz = "America/New_York"))
+  time <- ymd("2021-04-01", tz = "America/New_York")
+  expect_equal(time - months(1), ymd("2021-03-01", tz = "America/New_York"))
+  time <- ymd("2022-05-01", tz = "America/New_York")
+  expect_equal(time - months(2), ymd("2022-03-01", tz = "America/New_York"))
+  time <- ymd_hms("2022-04-02 04:01:01", tz = "America/New_York")
+  expect_equal(time - period("1m 1d 4H 1M 1S"), ymd("2022-03-01", tz = "America/New_York"))
+})
 
 ## ## bug #319
 ## x <- structure(list(sec = 0, min = 0, hour = 0, mday = -212, mon = 7L,
